
------------------------------
-- Containers
------------------------------

tiles = {}

------------------------------
-- Misc Variables
------------------------------

local screen =  {love.window.getWidth(), love.window.getHeight()}
local W_half, H_half = love.window.getWidth()/2, love.window.getHeight()/2

light_position = {1, 1, 0}
object_position = {}
tile_size = 128

------------------------------




---------------------------------------------------
-- Tile class
---------------------------------------------------
Tile = {}
Tile.__index = Tile

-- Class methods
function Tile.init(subtype, x, y, extra)
	local self = {}

	--Basic Attributes
	self.subtype = subtype
	self.x, self.y, self.z = x, y, 0
	self.size = {tile_size, tile_size}
	self.texture = tile_tex

	-- Shadow Attributes
	self.shadowTexture = tile_tex

	self.shadowCoords = {
								{0, 0, 0, 0},
								{self.size[1], 0, 1, 0},
								{self.size[1], self.size[2], 1, 1},
								{0, self.size[2], 0, 1}
							}


	self.shadowGeometry =  love.graphics.newMesh(self.shadowCoords, self.shadowTexture)


	----------------
	setmetatable(self, Tile)
	return self
end



function Tile.render(self)

	--DRAW SHADOWS

	-- Set color to black
	love.graphics.setColor(0, 0, 0, 255)
	love.graphics.setBlendMode("alpha")

	-- Set Distortion shader
	love.graphics.setShader(effect)

	-- Send Shader Info
	effect:send('light_position', {light_position[1], light_position[2], light_position[3]})
	effect:send('object_position', {self.x, self.y, 0})

	--TOP SHADOW
	effect:send('free_edge', {0, tile_size, tile_size, tile_size})
	effect:send('fixed_edge', {0, 0, tile_size, 0})
	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)

	--RIGHT SHADOW
	effect:send('free_edge', {0, 0, 0, tile_size})
	effect:send('fixed_edge', {tile_size, 0, tile_size, tile_size})
	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)

	--BOTTOM SHADOW
	effect:send('free_edge', {0, 0, tile_size, 0})
	effect:send('fixed_edge', {0, tile_size, tile_size, tile_size})
	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)

	--LEFT SHADOW
	effect:send('free_edge', {tile_size, 0, tile_size, tile_size})
	effect:send('fixed_edge', {0, 0, 0, tile_size})
	love.graphics.draw(self.shadowGeometry, self.x, self.y, 0, 1, 1)


	love.graphics.setShader()


	-- DRAW OBJECT GRAPHICS
	love.graphics.setColor(150, 150, 150, 255)
	love.graphics.draw(self.texture, self.x, self.y)

end
---------------------------------------------------



function love.load()

	--LOAD SHADERS
	effect = love.graphics.newShader('effect.glsl')

	-- Load images
	tile_tex = love.graphics.newImage("tile.png")
	ball_tex = love.graphics.newImage("ball_64.png")

	-- Create Tiles
	tiles[#tiles+1] = Tile.init('basic', 384, 128)

end



-- QUIT FUNCTION
function love.keyreleased(key)

   if key == "escape" then
		love.event.push("quit")
	end

end


-- UPDATE STUFF
function love.update(dt)

	light_position[1] = love.mouse.getX()
	light_position[2] = love.mouse.getY()

	-- move tiles
	for i =1, #tiles do

		if love.keyboard.isDown("w") then
			tiles[i].y = tiles[i].y - 200 *dt

		elseif love.keyboard.isDown("s") then
			tiles[i].y = tiles[i].y + 200 *dt

		end


		if love.keyboard.isDown("a") then
			tiles[i].x = tiles[i].x - 200 *dt

		elseif love.keyboard.isDown("d") then
			tiles[i].x = tiles[i].x + 200 *dt

		end

	end

	object_position = {tiles[1].x/screen[1], tiles[1].y/screen[2], 0}

end



-- DRAW STUFF
function love.draw()

	love.graphics.setBackgroundColor(150, 150, 150)
	love.graphics.clear( )

	--Draw Tiles
	for tile = 1, #tiles do

		local object = tiles[tile]

		object:render()

	end


	-- Draw ball at light position
	love.graphics.setColor(0, 255, 0, 255)
	love.graphics.draw(ball_tex, light_position[1], light_position[2], 0, 0.25, 0.25, 32, 32)

	------------------------
	--Debug Info
	------------------------
	love.graphics.setColor(255, 255, 100, 255)
	love.graphics.setBlendMode("alpha")

	love.graphics.print("tile: ".."x:"..tiles[1].x, 300, 10)
	love.graphics.print("tile: ".."y:"..tiles[1].y, 300, 20)

	love.graphics.print("light: ".."x:"..light_position[1], 600, 10)
	love.graphics.print("light: ".."y:"..light_position[2], 600, 20)

	love.graphics.print("Current FPS: "..tostring(love.timer.getFPS( )), 10, 10)

end
